/**
 * @file globals.h 
 * 
 * Global definitions
 * 
 * @author Tecnologix, 2015
 */
#ifndef GLOBALS_H
#define GLOBALS_H

#include "stm32f746xx.h"
#include "arm_math.h"
#include <stdbool.h>
#include <stdint.h>

/* FIR WS filter coefficients number */
#define FIR_WS_NUM_TAPS          (128)

/* FIR MA filter coefficients number */
#define FIR_MA_NUM_TAPS          (10)

/* ====================================================================================================
 * MACROS
 * ==================================================================================================== */
 
/* Audio block size, in half words.
   The whole recorded buffer is composed of AUDIO_N_BLOCKS*AUDIO_BLOCK_SIZE half words.
   The left stream is stored at the odd indexes, the right strem at the even ones */
#define AUDIO_BLOCK_SIZE      ((uint32_t)0xFFFE) 

/* Number of audio blocks */
#define AUDIO_N_BLOCKS        ((uint32_t)2)

/* Currently played audio buffer (helf words, 512 per channel) */
#define AUDIO_BUFFER_SIZE     (1024)

/* Number of samples to filter */
#define NUM_SAMPLES           (AUDIO_BUFFER_SIZE/4)

/* Filtered stream size  (half word) */
#define FILTERED_STREAM_SIZE  (AUDIO_BUFFER_SIZE/2)

/* Number of samples that must be shown on graph, PER stream */
#define GRAPH_NUM_SAMPLES     (FILTERED_STREAM_SIZE)

/* Audio playing volume */
#define AUDIO_VOLUME          (70)

/* Audio frequency */
#define AUDIO_FREQUENCY       (DEFAULT_AUDIO_IN_FREQ)

#define IIR_N_STAGES          (2)

#define N_CHANNELS            (2)
#define LEFT_CHANNEL          (0)
#define RIGHT_CHANNEL         (1)

/* ====================================================================================================
 * TYPE DEFINITIONS
 * ==================================================================================================== */

/* Buffert state */
typedef enum {
  BUFFER_OFFSET_NONE = 0,  
  BUFFER_OFFSET_HALF,  
  BUFFER_OFFSET_FULL,     
} buffer_state_t;

/* The playback buffer */
typedef struct {
  /* The buffer currently played */
  uint16_t buff[AUDIO_BUFFER_SIZE];
  /* The reference within the stream file */
  uint32_t fptr;  
  /* The buffer DMA state */
  buffer_state_t state;
} audio_buffer_t;

/* Filter type */
typedef enum
{
  WINDOWED_SINC       =0,
  MOVING_AVERAGE,
  IIR_BIQUAD_F32,
  IIR_BIQUAD_Q15,
  NO_FILTER
} filter_type_t;

/* This structure keeps trace of the user choices */
typedef struct
{
  /* Enable left channel */
  bool enable_left;
  
  /* Enable right channel */
  bool enable_right;
  
  /* Enabled filters */
  bool enabled_filters[4];
  
  /* Enable FFT */
  bool enable_fft;
} menu_t;

/* ====================================================================================================
 * VARIABLES EXTERNALIZATION
 * ==================================================================================================== */

/* The left stream */
extern const uint8_t l_stream[131068U];

/* The right stream */
extern const uint8_t r_stream[131068U];

/* The playback buffer */
extern audio_buffer_t buffer_ctl;

/* Filtered left stream */
extern q15_t filtered_left_stream[FILTERED_STREAM_SIZE];

/* Filtered rigth stream */
extern q15_t filtered_right_stream[FILTERED_STREAM_SIZE];

/* Left stream graph data */
extern short lcd_left[GRAPH_NUM_SAMPLES];

/* Right stream graph data */
extern short lcd_right[GRAPH_NUM_SAMPLES];

/* --------------------------------------------------------------------------------------------------
 * MENU
 * -------------------------------------------------------------------------------------------------- */

/* The menu */
extern menu_t menu;

/* --------------------------------------------------------------------------------------------------
 * TIME MEASURE
 * -------------------------------------------------------------------------------------------------- */

/* dsp_time_end - dsp_time_start gives an approximated measure about the time spent in DSP operations.
   It is approximated because of operating system, lcd interrupts, audio transfer interrupts */ 
extern uint32_t dsp_time_start;
extern uint32_t dsp_time_end;
extern uint32_t dsp_time_diff;

/* --------------------------------------------------------------------------------------------------
 * REAL FFT 
 * -------------------------------------------------------------------------------------------------- */

/* The real FFT operator */
extern arm_rfft_instance_q15 rfft;

/* The Real FFT output buffer */
extern q15_t rfft_out_buffer[2*NUM_SAMPLES];

/* The Real FFT magnitude buffer */
extern float32_t rfft_mag_buffer[2*NUM_SAMPLES];

/* --------------------------------------------------------------------------------------------------
 * cURRENT FILTER
 * -------------------------------------------------------------------------------------------------- */

/* Filter */
extern filter_type_t filter_type;

/* --------------------------------------------------------------------------------------------------
 * FIR WINDOWED SINC FILTER
 * -------------------------------------------------------------------------------------------------- */

/* The left filter */
extern arm_fir_instance_q15 left_fir_ws;

/* The right filter */
extern arm_fir_instance_q15 right_fir_ws;

/* Left filter state buffer */
extern q15_t left_filter_ws_state_buffer[FIR_WS_NUM_TAPS + NUM_SAMPLES];

/* Right filter state buffer */
extern q15_t right_filter_ws_state_buffer[FIR_WS_NUM_TAPS + NUM_SAMPLES];

/* Filter coefficients float */
extern const float32_t fir_ws_coeff_float[FIR_WS_NUM_TAPS];

/* Filter coefficients Q15 */
extern q15_t fir_ws_coeff[FIR_WS_NUM_TAPS];

/* --------------------------------------------------------------------------------------------------
 * FIR MOVING AVERAGE FILTER
 * -------------------------------------------------------------------------------------------------- */

/* The left filter */
extern arm_fir_instance_q15 left_fir_ma;

/* The right filter */
extern arm_fir_instance_q15 right_fir_ma;

/* Left filter state buffer */
extern q15_t left_filter_ma_state_buffer[FIR_MA_NUM_TAPS + NUM_SAMPLES];

/* Right filter state buffer */
extern q15_t right_filter_ma_state_buffer[FIR_MA_NUM_TAPS + NUM_SAMPLES];

/* Filter coefficients float */
extern const float32_t fir_ma_coeff_float[FIR_MA_NUM_TAPS];

/* Filter coefficients Q15 */
extern q15_t fir_ma_coeff[FIR_MA_NUM_TAPS];

/* --------------------------------------------------------------------------------------------------
 * IIR F32 FILTER
 * -------------------------------------------------------------------------------------------------- */

/* The left IIR filter */
extern arm_biquad_casd_df1_inst_f32 left_iir_f32;

/* The right IIR filter */
extern arm_biquad_casd_df1_inst_f32 right_iir_f32;

/* Left filter state buffer */
extern float32_t left_iir_filter_f32_state_buffer[4*IIR_N_STAGES];

/* Right filter state buffer */
extern float32_t right_iir_filter_f32_state_buffer[4*IIR_N_STAGES];

/* Auxiliary left float buffer */
extern float32_t left_aux_float_buffer[FILTERED_STREAM_SIZE/2];

/* Auxiliary right float buffer */
extern float32_t right_aux_float_buffer[FILTERED_STREAM_SIZE/2];

/* Filter coefficients */
extern const float32_t iir_f32_coeff_float[5*IIR_N_STAGES];

/* --------------------------------------------------------------------------------------------------
 * IIR Q15 FILTER
 * -------------------------------------------------------------------------------------------------- */

/* The left IIR filter */
extern arm_biquad_casd_df1_inst_q15 left_iir_q15;

/* The right IIR filter */
extern arm_biquad_casd_df1_inst_q15 right_iir_q15;

/* Left filter state buffer */
extern q15_t left_iir_filter_q15_state_buffer[4*IIR_N_STAGES];

/* Right filter state buffer */
extern q15_t right_iir_filter_q15_state_buffer[4*IIR_N_STAGES];

/* IIR Q15 filter coefficients */
extern q15_t iir_filter_q15_coeff[6*IIR_N_STAGES];

/* Filter coefficients */
extern const float32_t iir_q15_coeff_float[6*IIR_N_STAGES];

extern q15_t iir_q15_coeff[6*IIR_N_STAGES];

#endif
