/**
 * @file globals.c
 * 
 * Global definitions
 * 
 * @author Tecnologix, 2015
 */
#include "globals.h"
#include "conf.h"
#include <stdbool.h>

/* ====================================================================================================
 * GLOBAL VARIABLES
 * ==================================================================================================== */

/* The playback buffer */
audio_buffer_t buffer_ctl;

/* Filtered left stream */
q15_t filtered_left_stream[FILTERED_STREAM_SIZE];

/* Filtered rigth stream */
q15_t filtered_right_stream[FILTERED_STREAM_SIZE];

/* Left stream graph data */
short lcd_left[GRAPH_NUM_SAMPLES];

/* Right stream graph data */
short lcd_right[GRAPH_NUM_SAMPLES];

/* --------------------------------------------------------------------------------------------------
 * MENU
 * -------------------------------------------------------------------------------------------------- */

/* The menu */
menu_t menu = {false, false, {false, false, false, false}, false};

/* --------------------------------------------------------------------------------------------------
 * TIME MEASURE
 * -------------------------------------------------------------------------------------------------- */

/* dsp_time_end - dsp_time_start gives an approximated measure about the time spent in DSP operations.
   It is approximated because of operating system, lcd interrupts, audio transfer interrupts */ 
uint32_t dsp_time_start;
uint32_t dsp_time_end;
uint32_t dsp_time_diff;

/* --------------------------------------------------------------------------------------------------
 * REAL FFT 
 * -------------------------------------------------------------------------------------------------- */

/* The real FFT operator */
arm_rfft_instance_q15 rfft;

/* The Real FFT output buffer 2* is due to complex and real part */
q15_t rfft_out_buffer[2*NUM_SAMPLES];

/* The Real FFT magnitude buffer */
float32_t rfft_mag_buffer[2*NUM_SAMPLES];

/* --------------------------------------------------------------------------------------------------
 * cURRENT FILTER
 * -------------------------------------------------------------------------------------------------- */

/* Filter */
filter_type_t filter_type = NO_FILTER;

/* --------------------------------------------------------------------------------------------------
 * FIR WINDOWED SINC FILTER
 * -------------------------------------------------------------------------------------------------- */

/* The left filter */
arm_fir_instance_q15 left_fir_ws;

/* The right filter */
arm_fir_instance_q15 right_fir_ws;

/* Left filter state buffer */
q15_t left_filter_ws_state_buffer[FIR_WS_NUM_TAPS + NUM_SAMPLES];

/* Right filter state buffer */
q15_t right_filter_ws_state_buffer[FIR_WS_NUM_TAPS + NUM_SAMPLES];

/* Filter coefficients Q15 */
q15_t fir_ws_coeff[FIR_WS_NUM_TAPS];

/* --------------------------------------------------------------------------------------------------
 * FIR MOVING AVERAGE FILTER
 * -------------------------------------------------------------------------------------------------- */

/* The left filter */
arm_fir_instance_q15 left_fir_ma;

/* The right filter */
arm_fir_instance_q15 right_fir_ma;

/* Left filter state buffer */
q15_t left_filter_ma_state_buffer[FIR_MA_NUM_TAPS + NUM_SAMPLES];

/* Right filter state buffer */
q15_t right_filter_ma_state_buffer[FIR_MA_NUM_TAPS + NUM_SAMPLES];

/* Filter coefficients Q15 */
q15_t fir_ma_coeff[FIR_MA_NUM_TAPS];

/* --------------------------------------------------------------------------------------------------
 * IIR F32 FILTER
 * -------------------------------------------------------------------------------------------------- */

/* The left IIR filter */
arm_biquad_casd_df1_inst_f32 left_iir_f32;

/* The right IIR filter */
arm_biquad_casd_df1_inst_f32 right_iir_f32;

/* Left filter state buffer */
float32_t left_iir_filter_f32_state_buffer[4*IIR_N_STAGES];

/* Right filter state buffer */
float32_t right_iir_filter_f32_state_buffer[4*IIR_N_STAGES];

/* Auxiliary left float buffer */
float32_t left_aux_float_buffer[FILTERED_STREAM_SIZE/2];

/* Auxiliary right float buffer */
float32_t right_aux_float_buffer[FILTERED_STREAM_SIZE/2];

/* --------------------------------------------------------------------------------------------------
 * IIR Q15 FILTER
 * -------------------------------------------------------------------------------------------------- */

/* The left IIR filter */
arm_biquad_casd_df1_inst_q15 left_iir_q15;

/* The right IIR filter */
arm_biquad_casd_df1_inst_q15 right_iir_q15;

/* Left filter state buffer */
q15_t left_iir_filter_q15_state_buffer[4*IIR_N_STAGES];

/* Right filter state buffer */
q15_t right_iir_filter_q15_state_buffer[4*IIR_N_STAGES];

/* IIR Q15 filter coefficients */
q15_t iir_filter_q15_coeff[6*IIR_N_STAGES];

/* Filter coefficients */
q15_t iir_q15_coeff[6*IIR_N_STAGES];

/* --------------------------------------------------------------------------------------------------
 * Include the filter coefficients (see conf.h)
 * -------------------------------------------------------------------------------------------------- */

/* FIR Windowed sinc */
#ifdef FIR_WS_ACTIVE_FILTER
  #if (FIR_WS_ACTIVE_FILTER==WINDOWED_SINC_LP_500HZ)
    #include "coeff_fir_ws_lp_500hz.h"
  #elif (FIR_WS_ACTIVE_FILTER==WINDOWED_SYNC_BP_1200_1700HZ)
    #include "coeff_fir_ws_bp_1300_1700hz.h"
  #else
    /* Extend the list adding your own coefficients herebelow */
  #endif
#else
  #error "You must define a windowed sinc filter"
#endif

/* FIR Moving Average */
#ifdef FIR_MA_ACTIVE_FILTER
  #if (FIR_MA_ACTIVE_FILTER==MOVING_AVG_10)
    #include "coeff_fir_ma_10.h"
  #else
    /* Extend the list adding your own coefficients herebelow */
  #endif
#else
  #error "You must define a moving average filter"
#endif


/* IIR F32 */
#ifdef IIR_F32_ACTIVE_FILTER
  #if (IIR_F32_ACTIVE_FILTER==IIR_BWORTH2_LP_500HZ)
    #include "coeff_iir_f32_lp_500Hz.h"
  #elif (IIR_F32_ACTIVE_FILTER==IIR_BWORTH2_HP_3000HZ)
    #include "coeff_iir_f32_hp_3000hz.h"
  #elif (IIR_F32_ACTIVE_FILTER==IIR_BWORTH2_HP_3500HZ)
    #include "coeff_iir_f32_hp_3500hz.h"
  #elif (IIR_F32_ACTIVE_FILTER==_IIR_BWORTH2_LP_500HZ)
    #include "coeff_iir_f32_lp_500hz.h"
  #else
    /* Extend the list adding your own coefficients herebelow */
  #endif
#else
  #error "You must define an IIR F32 filter"
#endif


/* IIR Q15 */
#ifdef IIR_Q15_ACTIVE_FILTER
  #if (IIR_Q15_ACTIVE_FILTER==IIR_BWORTH2_LP_500HZ)
    #include "coeff_iir_q15_lp_500Hz.h"
  #elif (IIR_Q15_ACTIVE_FILTER==IIR_BWORTH2_HP_3500HZ)
    #include "coeff_iir_q15_hp_3500hz.h"
  #else
    /* Extend the list adding your own coefficients herebelow */
  #endif
#else
  #error "You must define an IIR Q15 filter"
#endif

