/**
 * @file lcd_env.c 
 * 
 * LCD environment function bodies implementation
 * 
 * @author Tecnologix, 2015
 */
#include "lcd_env.h"
#include "GUI.h"
#include "WM.h"
#include "GRAPH.h" 
#include "DIALOG.h"
#include "globals.h"
#include "conf.h"
#include <stdint.h>
#include <stdio.h>

/* ===============================================================================
 * MACROS
 * =============================================================================== */

/* Frame size */
#define FRAME_XSIZE               (480)
#define FRAME_YSIZE               (272)

/* Graph widget attributes */
#define GRAPH_START_X             (5)
#define GRAPH_START_Y             (5)

/* Checkboxes */
#define CHECKBOX_SIZE             (60)

#define MENU_Y_POS                (230)

/* Left channel checkbox */
#define CHK_L_XPOS                (5)

/* Left channel checkbox */
#define CHK_R_XPOS                (60)

/* FIR Filter checkbox */
#define CHK_FIR_WS_XPOS           (120)

#define CHK_FIR_MA_XPOS           (180)

/* IIR F32 Filter checkbox */
#define CHK_IIR_F32_XPOS          (260)

#define CHK_IIR_Q15_XPOS          (320)


/* FFT checkbox */
#define CHK_FFT_XPOS              (420)

/* Number of widgets ti create */
#define DLG_ARRAY_SIZE            (18)

/* ===============================================================================
 * PRIVATE VARIABLES
 * =============================================================================== */
 
 
/* The array to create the dialog window and its widgets */
static const GUI_WIDGET_CREATE_INFO create_dlg_array[] = {
  { FRAMEWIN_CreateIndirect, "STMicroelectronics - Keil - Tecnologix DEMO",  0,   0,   0, FRAME_XSIZE, FRAME_YSIZE, FRAMEWIN_CF_MAXIMIZED},
  { GRAPH_CreateIndirect, 0, GUI_ID_GRAPH0, GRAPH_START_X, GRAPH_START_Y, GRAPH_X_SIZE, GRAPH_Y_SIZE},
  { CHECKBOX_CreateIndirect,        0,  GUI_ID_CHECK0, CHK_L_XPOS, MENU_Y_POS,  CHECKBOX_SIZE,   0},
  { CHECKBOX_CreateIndirect,        0,  GUI_ID_CHECK1, CHK_R_XPOS, MENU_Y_POS,  CHECKBOX_SIZE,   0},
  { CHECKBOX_CreateIndirect,        0,  GUI_ID_CHECK2, CHK_FIR_WS_XPOS, MENU_Y_POS,  CHECKBOX_SIZE,   0},
  { CHECKBOX_CreateIndirect,        0,  GUI_ID_CHECK3, CHK_IIR_F32_XPOS, MENU_Y_POS,  CHECKBOX_SIZE,   0},
  { CHECKBOX_CreateIndirect,        0,  GUI_ID_CHECK4, CHK_FFT_XPOS, MENU_Y_POS,  CHECKBOX_SIZE,   0},
  { CHECKBOX_CreateIndirect,        0,  GUI_ID_CHECK5, CHK_FIR_MA_XPOS, MENU_Y_POS,  CHECKBOX_SIZE,   0},
  { CHECKBOX_CreateIndirect,        0,  GUI_ID_CHECK6, CHK_IIR_Q15_XPOS, MENU_Y_POS,  CHECKBOX_SIZE,   0},
  { TEXT_CreateIndirect, "L",       0 , CHK_L_XPOS+20, MENU_Y_POS, 20, 35},
  { TEXT_CreateIndirect, "R",       0 , CHK_R_XPOS+20, MENU_Y_POS, 20, 35},
  { TEXT_CreateIndirect, "FIR-WS",  0 , CHK_FIR_WS_XPOS+20, MENU_Y_POS, 40, 35},
  { TEXT_CreateIndirect, "FIR-MA",  0 , CHK_FIR_MA_XPOS+20, MENU_Y_POS, 40, 35},
  { TEXT_CreateIndirect, "IIR-F32", 0 , CHK_IIR_F32_XPOS+20, MENU_Y_POS, 40, 35},
  { TEXT_CreateIndirect, "IIR-Q15", 0 , CHK_IIR_Q15_XPOS+20, MENU_Y_POS, 40, 35},
  { TEXT_CreateIndirect, "FFT",     0 , CHK_FFT_XPOS+20, MENU_Y_POS, 20, 35},
  { TEXT_CreateIndirect, "Hz (x1000)",  GUI_ID_TEXT0, GRAPH_X_SIZE/2, GRAPH_Y_SIZE - 40, 100, 35},
  { TEXT_CreateIndirect, "",            GUI_ID_TEXT1, 360, 10, 120, 35},
};


/* The dialog item */
static WM_HWIN dialog_item;

/* The graph item handler */
static WM_HWIN graph_item;

/* Left and rigth data handles */
static GRAPH_DATA_Handle left_data_handle, right_data_handle;

/* The graph horizontal scale handle */
static GRAPH_SCALE_Handle hor_scale_handle;   
static GRAPH_SCALE_Handle fft_hor_scale_handle;   

/* True when the FFT hor scale has to be shown */
static bool show_fft_hor_scale = false;

/* The filters checkboxes */
static uint16_t filter_checkboxes[4]={GUI_ID_CHECK2, GUI_ID_CHECK5, GUI_ID_CHECK3, GUI_ID_CHECK6};

/* Variable text boxes */
static WM_HWIN  fft_label_item;
static WM_HWIN  dsp_time_item;

/* Where the DSP time will be written */
static char dsp_time[50];

/* ===============================================================================
 * PRIVATE FUNCTIONS
 * =============================================================================== */

/* Enable the given filter */
static void enable_filter(WM_HWIN  dlg, filter_type_t enabled_filter)
{
  uint8_t i;
  
  menu.enabled_filters[enabled_filter] = CHECKBOX_GetState(WM_GetDialogItem(dlg, filter_checkboxes[enabled_filter]));
              
  if (menu.enabled_filters[enabled_filter])
  {
    filter_type = enabled_filter;
    
    /* Mutual exclusion - Disable all the other */
    for(i=0; i<4; i++)
    {
      if (i!=enabled_filter)
      {
        CHECKBOX_SetState(WM_GetDialogItem(dlg, filter_checkboxes[i]), 0);
      }
    }
  }
  else
  {
    if (filter_type == enabled_filter)
    {
      filter_type = NO_FILTER;
    }
  }
}

/* Dialog process callback */
static void wm_callback(WM_MESSAGE * msg){ 
  int      n_code;
  int      id;
  WM_HWIN  dlg;
  WM_HWIN  check_left_item;
  WM_HWIN  check_right_item;
  
  /* Get the dialog */
  dlg = msg->hWin;
  
  /* Switch on message id */
  switch (msg->MsgId) 
  {
    /* Initialize the dialog window */
    case WM_INIT_DIALOG:
      /* Get the dialog item */
      graph_item = WM_GetDialogItem(dlg, GUI_ID_GRAPH0);
  
      /* Creates the data handles */
      left_data_handle  = GRAPH_DATA_YT_Create(GUI_RED, GRAPH_X_SIZE, lcd_left, GRAPH_X_SIZE);
      right_data_handle = GRAPH_DATA_YT_Create(GUI_GREEN, GRAPH_X_SIZE, lcd_right, GRAPH_X_SIZE);
    
      /* Attach handles */
      GRAPH_AttachData(graph_item, left_data_handle);
      GRAPH_AttachData(graph_item, right_data_handle);
    
      GRAPH_DATA_YT_SetAlign(left_data_handle, GRAPH_ALIGN_LEFT );
      GRAPH_DATA_YT_SetAlign(right_data_handle, GRAPH_ALIGN_LEFT );
         
      /* Set graph attributes */
      GRAPH_SetGridDistY(graph_item, 25);
      GRAPH_SetGridVis(graph_item, 1);
      GRAPH_SetGridFixedX(graph_item, 1);
         
      /* Create and add horizontal scale (number of samples) */
      hor_scale_handle = GRAPH_SCALE_Create(GRAPH_Y_SIZE - 20, GUI_TA_HCENTER, GRAPH_SCALE_CF_HORIZONTAL, 50);
      GRAPH_SCALE_SetTextColor(hor_scale_handle, GUI_MAGENTA);
      GRAPH_AttachScale(graph_item, hor_scale_handle);
      
      /* Create and add horizontal scale for FFT (number of samples) */
      fft_hor_scale_handle = GRAPH_SCALE_Create(GRAPH_Y_SIZE - 55, GUI_TA_LEFT, GRAPH_SCALE_CF_HORIZONTAL, 128);
      GRAPH_SCALE_SetTextColor(fft_hor_scale_handle, GUI_CYAN);
      GRAPH_SCALE_SetOff(fft_hor_scale_handle, GRAPH_X_SIZE/2);
      GRAPH_SCALE_SetFactor(fft_hor_scale_handle, (2.0/32.0));
      GRAPH_SCALE_SetTickDist(fft_hor_scale_handle, 128/4);
      
      /* Get the FFT label item */
      fft_label_item = WM_GetDialogItem(dlg, GUI_ID_TEXT0);
      TEXT_SetTextColor(fft_label_item, GUI_CYAN);
      WM_HideWindow(fft_label_item);
      
      /* Get the DSP time item */
      dsp_time_item = WM_GetDialogItem(dlg, GUI_ID_TEXT1);
      TEXT_SetTextColor(dsp_time_item, GUI_YELLOW);
      
      /* Set the checkboxes */
      check_left_item = WM_GetDialogItem(dlg, GUI_ID_CHECK0);      
      check_right_item = WM_GetDialogItem(dlg, GUI_ID_CHECK1);      
      CHECKBOX_SetState(check_left_item, 1);
      CHECKBOX_SetState(check_right_item, 1);
    
      WM_SetStayOnTop(graph_item, 1);
      break;
      
    /* Notify parent */
    case WM_NOTIFY_PARENT:
      /* Id of the widget */
      id    = WM_GetId(msg->hWinSrc);      
      /* Notification code */
      n_code = msg->Data.v;      
      /* Switch on notification code */
      switch(n_code)
      {
        /* Check checkbozes values */
        case WM_NOTIFICATION_VALUE_CHANGED:
          switch(id)
          {
            /* Left channel enable */
            case GUI_ID_CHECK0:
              menu.enable_left = !menu.enable_left;
              break;
            
            /* Right channel enable */
            case GUI_ID_CHECK1:
              menu.enable_right = !menu.enable_right;
              break;
            
            /* Enable Windowed Sinc Filter */
            case GUI_ID_CHECK2:
              enable_filter(dlg,WINDOWED_SINC);
              break;
              
            /* Enable IIR F32 filter */  
            case GUI_ID_CHECK3:
              enable_filter(dlg,IIR_BIQUAD_F32);
              break;
            
            /* Enable IIR F32 filter */  
            case GUI_ID_CHECK5:
              enable_filter(dlg,MOVING_AVERAGE);
              break;
            
            /* Enable IIR F32 filter */  
            case GUI_ID_CHECK6:
              enable_filter(dlg,IIR_BIQUAD_Q15);
              break;
              
            /* Enable iir q15 filter */
            case GUI_ID_CHECK4:
              menu.enable_fft = !menu.enable_fft;
              break;
              
            
            /* Never reached */
            default:
              break;
          }
          break;
          
        /* Never reached */
        default:
          break;
      }
      break;
    
    default:
      WM_DefaultProc(msg);
  }
}

/* Data scaling */
static void lcd_graph_scale(short * pDataIn, short * pDataOut, int32_t inMax, int32_t outMax, uint32_t numSamples, int16_t offset, uint16_t trans)
{
  uint32_t i,j;
  
  for(i=0; i< trans; i++)
  {
    pDataOut[i] = offset + outMax/2; 
  }
	
	for (j=0; j < numSamples; j++, i++)
	{
		pDataOut[i] = offset + ((((int32_t)(pDataIn[j]) + inMax/2) * outMax)/inMax);		  
	}
  
  for(; i<GRAPH_X_SIZE; i++)
  {
    pDataOut[i] = offset + outMax/2; 
  }
}

/* ===============================================================================
 * PUBLIC FUNCTIONS
 * =============================================================================== */

/* 
 * graph_init
 * See header file for details
 */
void lcd_graph_init(void)
{
  WM_SetCreateFlags(WM_CF_MEMDEV);
  
  /* Use more buffers */
  WM_MULTIBUF_Enable(1);
  
  /* Create the dialog */
  dialog_item = GUI_CreateDialogBox(create_dlg_array, DLG_ARRAY_SIZE, wm_callback, 0, 0, 0);
  
  /* Avoid compiler warnings */
  dialog_item = dialog_item;
}  

/*
 * graph_show
 * See header file for details
 */
void lcd_graph_show(short * pData1_in, short * pData1_out, 
                    short * pData2_in, short * pData2_out, 
                    uint16_t d1_scale, uint16_t d2_scale, 
                    uint16_t d1_n_data, uint16_t d2_n_data,
                    uint16_t left_trans, uint16_t right_trans)
{ 
  if (menu.enable_fft)
  {
    if (!show_fft_hor_scale)
    {
      show_fft_hor_scale = true;
      GRAPH_AttachScale(graph_item, fft_hor_scale_handle);
      WM_ShowWindow(fft_label_item);
    }
  }
  else
  {
    if (show_fft_hor_scale)
    {
      show_fft_hor_scale = false;
      GRAPH_DetachScale(graph_item, fft_hor_scale_handle);
      WM_HideWindow(fft_label_item);
    }
  }
  
  sprintf(dsp_time,"Filtering time=%d", dsp_time_diff);
  TEXT_SetText(dsp_time_item, dsp_time);
  
  /* Detach old data */
  GRAPH_DetachData(graph_item, left_data_handle);
  GRAPH_DetachData(graph_item, right_data_handle);
  
  /* Delete old data handlers */
  GRAPH_DATA_YT_Delete(left_data_handle);
  GRAPH_DATA_YT_Delete(right_data_handle);
  
  lcd_graph_scale (pData1_in, pData1_out, d1_scale, GRAPH_Y_SIZE, d1_n_data, 50, left_trans);		
  lcd_graph_scale (pData2_in, pData2_out, d2_scale, GRAPH_Y_SIZE, d2_n_data, -50, right_trans);

  /* Create new data handlers */
  left_data_handle  = GRAPH_DATA_YT_Create(GUI_RED, GRAPH_X_SIZE, lcd_left, GRAPH_X_SIZE);
  right_data_handle = GRAPH_DATA_YT_Create(GUI_GREEN, GRAPH_X_SIZE, lcd_right, GRAPH_X_SIZE);

  /* Attach new data handler */
  GRAPH_AttachData(graph_item, left_data_handle);
  GRAPH_AttachData(graph_item, right_data_handle);
  
  GRAPH_DATA_YT_SetAlign(left_data_handle, GRAPH_ALIGN_LEFT );
  GRAPH_DATA_YT_SetAlign(right_data_handle, GRAPH_ALIGN_LEFT );
}
