/**
 * @file main.c 
 * 
 * Audio player application
 * 
 * @author Tecnologix, 2015
 */
#include "stm32f7xx_hal.h"
#include "stm32746g_discovery_audio.h"
#include "stm32746g_discovery_sdram.h"
#include "globals.h"
#include "lcd_env.h"
#include "GUI.h"
#include "DIALOG.h"
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include "cmsis_os.h"
#include "conf.h"

/* ====================================================================================================
 * MACROS
 * ==================================================================================================== */


#ifdef RTE_CMSIS_RTOS_RTX
extern uint32_t os_time;

uint32_t HAL_GetTick(void) { 
  return os_time; 
}
#endif

#define DSP_TIME_START  dsp_time_start = tim2.Instance->CNT
#define DSP_TIME_END    dsp_time_end = tim2.Instance->CNT;             \
                        dsp_time_diff = dsp_time_end - dsp_time_start

/* ====================================================================================================
 * TYPE DEFINITIONS
 * ==================================================================================================== */

/* This contains all the filters instances */
typedef struct
{
  /* Windowed Sinc FIR filter */
  arm_fir_instance_q15          *fir_windowed_sinc[N_CHANNELS];
  
  /* Moving Average FIR filter */
  arm_fir_instance_q15          *fir_moving_avg[N_CHANNELS];
  
  /* IIR Biquad Cascade (Butterworth) float32 */
  arm_biquad_casd_df1_inst_f32  *iir_f32[N_CHANNELS];
  
  /* IIR Biquad Cascade (Butterworth) Q15 */
  arm_biquad_casd_df1_inst_q15  *iir_q15[N_CHANNELS];
  
  /* F32 auxiliary buffers (used only for IIR F32 filters) */
  float32_t                     *aux_f32_buffers[N_CHANNELS];
} channels_t;

/* ====================================================================================================
 * PRIVATE VARIABLES
 * ==================================================================================================== */

/* The filter instances */
static channels_t channels = 
{
  {&left_fir_ws, &right_fir_ws},
  {&left_fir_ma, &right_fir_ma},
  {&left_iir_f32, &right_iir_f32},
  {&left_iir_q15, &right_iir_q15},
  {left_aux_float_buffer, right_aux_float_buffer}
};

/* The size of a single stream (helf words) */
static uint32_t stream_size;

/* True when new data to show are available */
static bool update_graph = false;

/* Keep trace on left and right channel */
static uint8_t channel_disabled[2] = {0, 0};

/* True when the LCD buffer has to be reset (before to applying the FFT) -- Graphic stuff */
static bool reset_lcd_buffer = true;

/* Timer 2 is used to measure the DSP time */
static TIM_HandleTypeDef tim2;

/* ====================================================================================================
 * PRIVATE FUNCTIONS
 * ==================================================================================================== */

/* 
 * System clock coonfiguration
 */
static void SystemClock_Config(void)
{
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_OscInitTypeDef RCC_OscInitStruct;
  
  /* Enable Power Control clock */
  __HAL_RCC_PWR_CLK_ENABLE();

  /* The voltage scaling allows optimizing the power consumption when the device is
     clocked below the maximum system frequency, to update the voltage scaling value 
     regarding system frequency refer to product datasheet.  */
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

  /* Enable HSE Oscillator and activate PLL with HSE as source */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 25;
  RCC_OscInitStruct.PLL.PLLN = 400;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 8;
  HAL_RCC_OscConfig(&RCC_OscInitStruct);

  /* activate the OverDrive to reach the 180 Mhz Frequency */
  HAL_PWREx_ActivateOverDrive();
 

  /* Select PLL as system clock source and configure the HCLK, PCLK1 and PCLK2
     clocks dividers */
  RCC_ClkInitStruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2);
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;

  HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_5);
}

/* 
 * Enable cache
 */
static void CPU_CACHE_ENABLE(void)
{
  (*(uint32_t *) 0xE000ED94) &= ~0x5;
  (*(uint32_t *) 0xE000ED98) = 0x0; //MPU->RNR
  (*(uint32_t *) 0xE000ED9C) = 0x20010000 |1<<4; //MPU->RBAR
  (*(uint32_t *) 0xE000EDA0) = 0<<28 | 3 <<24 | 0<<19 | 0<<18 | 1<<17 | 0<<16 | 0<<8 | 30<<1 | 1<<0 ; //MPU->RASE  WT
  (*(uint32_t *) 0xE000ED94) = 0x5;

  /* Invalidate I-Cache : ICIALLU register*/
  SCB_InvalidateICache();	
	
  /* Enable branch prediction */
  SCB->CCR |= (1 <<18); 
  __DSB();

  /* Enable I-Cache */
  SCB_EnableICache();	
	
  /* Enable D-Cache */
  SCB_InvalidateDCache();
  SCB_EnableDCache();
}

/* Disable channel. n_data is measured in BYTES */
static void disable_channel(uint16_t *channel, uint32_t n_data, uint8_t channel_id)
{
  if (channel_disabled[channel_id]<2)
  {
    memset(channel,0,n_data);
    channel_disabled[channel_id]++;
  }
}

/* Set the lcd buffer of the right channel to zero */
static void reset_lcd_right_buffer(void)
{
  uint16_t i;
  
  if (reset_lcd_buffer)
  {
    for(i=0;i<GRAPH_X_SIZE; i++)
    {
      lcd_left[i] = -50 + GRAPH_Y_SIZE/2;
    }
       
    reset_lcd_buffer = false;
  }
}

/**
 * Filters the given channel
 */
static void filter_channel(uint32_t channel, q15_t *in_data_stream, q15_t *out_data_stream, uint32_t n_data)
{
  float32_t *aux_float_buffer;
    
  if (filter_type==NO_FILTER)
  {
    dsp_time_diff = 0;
    memcpy(out_data_stream, in_data_stream, 2*n_data);
  }
  else
  {
    /* Switch on filter type */
    switch(filter_type)
    { 
      /* FIR Moving Average Filter */
      case MOVING_AVERAGE:
        DSP_TIME_START;
        
        arm_fir_q15(channels.fir_moving_avg[channel], in_data_stream, out_data_stream, n_data);
        
        DSP_TIME_END;
        break;
      
      /* FIR Windowed Sinc */
      case WINDOWED_SINC:
        DSP_TIME_START;
      
        arm_fir_q15(channels.fir_windowed_sinc[channel], in_data_stream, out_data_stream, n_data);
      
        DSP_TIME_END;
        break;
      
      /* IIR Biquad Cascade F32 */
      case IIR_BIQUAD_F32:
        aux_float_buffer = channels.aux_f32_buffers[channel];  
        
        arm_q15_to_float (in_data_stream, aux_float_buffer, n_data); 
        
        DSP_TIME_START;
      
        arm_biquad_cascade_df1_f32 (channels.iir_f32[channel], aux_float_buffer, aux_float_buffer, n_data);
      
        DSP_TIME_END;
        
        arm_float_to_q15 (aux_float_buffer, out_data_stream, n_data); 
        break;
      
      /* IIR Biquad Cascade Q15 */
      case IIR_BIQUAD_Q15:
        DSP_TIME_START;
        
        arm_biquad_cascade_df1_q15 (channels.iir_q15[channel], in_data_stream, out_data_stream, n_data);
      
        DSP_TIME_END;
      
        if (IIR_Q15_POST_SHIFT)
        {
          arm_shift_q15(out_data_stream,-2*IIR_Q15_POST_SHIFT,out_data_stream,n_data);
        }
        break;
      
      /* Never reached */
      default:
        break;
    }
  }
}

/* Do FFT */
static void do_fft(q15_t *in_data_stream, q15_t *out_data_stream, uint32_t n_data)
{ 
  arm_rfft_q15 (&rfft, in_data_stream, out_data_stream);
  
  arm_q15_to_float (out_data_stream, rfft_mag_buffer, 2*n_data); 
  
  arm_cmplx_mag_f32 (rfft_mag_buffer, rfft_mag_buffer, n_data);
  
  arm_float_to_q15 (rfft_mag_buffer, out_data_stream, n_data);
}

/** 
 * @param ldata   - Left stream
 * @param rdata   - Right stream
 * @param offset  - offset within the streams
 * @param pbuf    - Pointer to the playback buffer
 * @param n_data  - Number of haf words to read FROM EACH stream
 * 
 * @return the number of half words read from each stream
 */
static uint32_t get_filtered_data(uint16_t *ldata,  uint16_t *rdata, uint32_t offset, uint16_t *pbuf, uint32_t n_data)
{
  /* Half words read from each stream */
  uint32_t data_read;
    
  /* Offset within the filtered stream */
  static uint16_t filtered_offset = 0;
  
  /* Filter Left channel */
  if (menu.enable_left)
  {
    /* Filter */
    filter_channel(LEFT_CHANNEL, (q15_t *)&ldata[offset], &filtered_left_stream[filtered_offset], n_data);
    channel_disabled[LEFT_CHANNEL] = 0;
  }
  else
  {
    disable_channel((uint16_t *)(&filtered_left_stream[filtered_offset]), 2*n_data, LEFT_CHANNEL);
  }
  
  /* Filter Right channel */
  if (menu.enable_right)
  {
    /* Filter */
    filter_channel(RIGHT_CHANNEL, (q15_t *)&rdata[offset], &filtered_right_stream[filtered_offset], n_data);
    channel_disabled[RIGHT_CHANNEL] = 0;
  }
  else
  {
    disable_channel((uint16_t *)(&filtered_right_stream[filtered_offset]), 2*n_data, RIGHT_CHANNEL);
  }
  
  /* Update playback buffer */
  data_read = 0;
  while(((offset + data_read) < stream_size) && (data_read < n_data))
  { 
    pbuf[2*data_read]     = filtered_left_stream[filtered_offset + data_read];
    pbuf[2*data_read + 1] = filtered_right_stream[filtered_offset + data_read];
    data_read++;    
  }
  
  /* Apply FFT on the right channel signal */
  if (menu.enable_fft)
  {
    reset_lcd_right_buffer();
    do_fft(&filtered_right_stream[filtered_offset], rfft_out_buffer, n_data);
  }
  else
  {
    reset_lcd_buffer = true;
  }
  
  /* Update offset in filtered buffer */
  filtered_offset += n_data;
  if (filtered_offset >= FILTERED_STREAM_SIZE)
  {
    filtered_offset = 0;
  }
    
  return data_read;
}


/** 
 * Starts audio
 * @param left  - The left stream
 * @param right - The right stream
 */
static void audio_start(uint16_t *left_stream, uint16_t *right_stream)
{
  uint32_t data_read;
  
  /* No bytes transferred yet */
  buffer_ctl.state = BUFFER_OFFSET_NONE;
  
  /* Transfer AUDIO_BUFFER_SIZE bytes to buffer_ctl.buff. Odd half word are get from left stream,
     while even half words are get from right stream */
  data_read = get_filtered_data(left_stream, right_stream, 0, &buffer_ctl.buff[0], AUDIO_BUFFER_SIZE/4);
  buffer_ctl.fptr = data_read;
  
  data_read = get_filtered_data(left_stream, right_stream, buffer_ctl.fptr, &buffer_ctl.buff[AUDIO_BUFFER_SIZE/2], AUDIO_BUFFER_SIZE/4);
  buffer_ctl.fptr += data_read;
  
  if(data_read > 0)
  {
    /* Play buffer */
    BSP_AUDIO_OUT_Play(&buffer_ctl.buff[0], 2*AUDIO_BUFFER_SIZE);
  }
}


/* 
 * Process audio
 */
static void audio_process(void)
{
  uint32_t data_read;
    
  if(buffer_ctl.fptr >= stream_size)
  {
    /* Play audio sample again ... */
    buffer_ctl.fptr = 0; 
  }

  /* 1st half buffer played; so fill it and continue playing from bottom*/
  if(buffer_ctl.state == BUFFER_OFFSET_HALF)
  {
      data_read = get_filtered_data((uint16_t *)l_stream, (uint16_t *)r_stream,
                          buffer_ctl.fptr,
                          &buffer_ctl.buff[0],
                          AUDIO_BUFFER_SIZE/4);
    
    if( data_read >0)
    { 
      buffer_ctl.state = BUFFER_OFFSET_NONE;
      buffer_ctl.fptr += data_read; 
    }
  }
  
  /* 2nd half buffer played; so fill it and continue playing from top */    
  if(buffer_ctl.state == BUFFER_OFFSET_FULL)
  {
      data_read = get_filtered_data((uint16_t *)l_stream, (uint16_t *)r_stream,
                        buffer_ctl.fptr, 
                        &buffer_ctl.buff[AUDIO_BUFFER_SIZE/2],
                        AUDIO_BUFFER_SIZE/4);
    if( data_read > 0)
    {
      buffer_ctl.state = BUFFER_OFFSET_NONE;
      buffer_ctl.fptr += data_read;
    }
    
    update_graph = true;
  }
}

/* Initialize the DSP timer */
static void init_dsp_timer(TIM_HandleTypeDef *tim)
{
  __HAL_RCC_TIM2_CLK_ENABLE();
  __HAL_RCC_TIMCLKPRESCALER(RCC_TIMPRES_ACTIVATED);
  tim->Instance = TIM2;
  tim->Instance->ARR = 0xFFFFFFFF;
  __HAL_TIM_ENABLE(tim);
}

/* ====================================================================================================
 * PUBLIC FUNCTIONS
 * ==================================================================================================== */

/* 
 * Application entry point
 */
int main(void)
{ 
  HAL_Init();
  CPU_CACHE_ENABLE();
  SystemClock_Config();
  BSP_SDRAM_Init();
  
  /* FILTERS INITIALIZATION  */
  
  /* FIR Windowed Sinc  -------------------------------------------- */ 
  
  /* Convert float coefficients to q15 coefficients */
  arm_float_to_q15 ((float32_t *)fir_ws_coeff_float, fir_ws_coeff, FIR_WS_NUM_TAPS);
   
  /* Initialize FIR Left Filter */
  arm_fir_init_q15 (&left_fir_ws, FIR_WS_NUM_TAPS, fir_ws_coeff, left_filter_ws_state_buffer, NUM_SAMPLES);
  
  /* Initialize FIR Right Filter */
  arm_fir_init_q15 (&right_fir_ws, FIR_WS_NUM_TAPS, fir_ws_coeff, right_filter_ws_state_buffer, NUM_SAMPLES);
  
  /* FIR Moving Average -------------------------------------------- */
  
  /* Convert float coefficients to q15 coefficients */
  arm_float_to_q15 ((float32_t *)fir_ma_coeff_float, fir_ma_coeff, FIR_MA_NUM_TAPS);
   
  /* Initialize FIR Left Filter */
  arm_fir_init_q15 (&left_fir_ma, FIR_MA_NUM_TAPS, fir_ma_coeff, left_filter_ma_state_buffer, NUM_SAMPLES);
  
  /* Initialize FIR Right Filter */
  arm_fir_init_q15 (&right_fir_ma, FIR_MA_NUM_TAPS, fir_ma_coeff, right_filter_ma_state_buffer, NUM_SAMPLES);
  
  /* IIR F32 -------------------------------------------- */
  
  /* Initialize IIR Left filter */
  arm_biquad_cascade_df1_init_f32(&left_iir_f32, IIR_N_STAGES, (float32_t *)iir_f32_coeff_float, left_iir_filter_f32_state_buffer);
    
  /* Initialize IIR Right filter */
  arm_biquad_cascade_df1_init_f32(&right_iir_f32, IIR_N_STAGES, (float32_t *)iir_f32_coeff_float, right_iir_filter_f32_state_buffer);
  
  /* IIR Q15 -------------------------------------------- */
  
  /* Convert coefficients to Q15 */
  arm_float_to_q15 ((float32_t *)iir_q15_coeff_float, iir_q15_coeff, 6*IIR_N_STAGES);
  
  /* Initialize IIR Left filter */
  arm_biquad_cascade_df1_init_q15(&left_iir_q15, IIR_N_STAGES, iir_q15_coeff, left_iir_filter_q15_state_buffer, IIR_Q15_POST_SHIFT);
    
  /* Initialize IIR Right filter */
  arm_biquad_cascade_df1_init_q15(&right_iir_q15, IIR_N_STAGES, iir_q15_coeff, right_iir_filter_q15_state_buffer, IIR_Q15_POST_SHIFT);
 
  /* REAL FAST FOURIER TRANSFORM ------------------------- */
  
  /* RFTT initialization */
  arm_rfft_init_q15(&rfft, NUM_SAMPLES, 0, 1);
  
  /* ----------------------------------------------------- */
 
  /* Initialize GUI */
  GUI_Init();
  
  /* Initialize DSP timer */
  init_dsp_timer(&tim2);
  
  /* Initialize graph */
  lcd_graph_init();
  
  /* Initialize stream_size (the length of a single stream in HALF WORDS) */
  stream_size = (AUDIO_BLOCK_SIZE * AUDIO_N_BLOCKS)/2;
          
  /* Playback audio */
  BSP_AUDIO_OUT_Init(OUTPUT_DEVICE_HEADPHONE, AUDIO_VOLUME, AUDIO_FREQUENCY);
  BSP_AUDIO_OUT_SetAudioFrameSlot(CODEC_AUDIOFRAME_SLOT_02);
  
  /* Starts audio playback */
  audio_start((uint16_t *)l_stream, (uint16_t *)r_stream);
  
  /* Main loop */
  while(1)
  {
    /* Process audio */
    audio_process();
    
    /* If data ready, update graph */
    if (update_graph) 
    {
      GUI_Exec();
      
      if (!menu.enable_fft)
      {
        lcd_graph_show(filtered_left_stream, lcd_left, 
                       filtered_right_stream, lcd_right,16384,16384, GRAPH_X_SIZE, GRAPH_X_SIZE, 0, 0);     
      }
      else
      { 
        /* Left centering - Half of lcd right plot is set to zero */
        lcd_graph_show(filtered_left_stream, lcd_left, rfft_out_buffer, lcd_right, 16384, 2048, GRAPH_X_SIZE, 0X80, 0, GRAPH_X_SIZE/2);     
      }
      
      update_graph = false;
    }
  }
}

